'''
AWS connection handlers
'''
import boto3.ec2
from configAWSEnv.prints import print_and_exit

def connect_to_ec2():
    try:
        ec2client = boto3.client('ec2')
    except Exception as e:
        print_and_exit("failed to connect to ec2, check your boto configuration\nException: {}".format(e), 2)
    return ec2client


def connect_to_ec2_resources():
    try:
        ec2resourcers = boto3.resource('ec2')
    except Exception as e:
        print_and_exit("failed to connect to ec2, check your boto configuration\nException: {}".format(e), 2)
    return ec2resourcers


'''
Main ASW functionality
- Get the list of instance to run on according to the AWS filter
- Get the required action (start or stop the instances)
- Run and verify state after instance status change.
- If fails - return AWS console URL with the list of problematic instances
'''

import boto3.ec2
from configAWSEnv.connection import connect_to_ec2_resources, connect_to_ec2
from configAWSEnv.const import WAKEUP, SHUTDOWN
from configAWSEnv.prints import print_and_exit
from configAWSEnv.parser import parse_args


def list_instances_by_tag_value(action, filters):

    # When passed a tag key, tag value this will return a list of InstanceIds that were found.
    # append instance state
    ec2client = connect_to_ec2()
    if action == WAKEUP:
        filters.append( {'Name': 'instance-state-name', 'Values': ['{}'.format('stopped')]})
    elif action == SHUTDOWN:
        filters.append( {'Name': 'instance-state-name', 'Values': ['{}'.format('running')]})

    response = ec2client.describe_instances(
        Filters=filters
    )

    instancelist = []
    for reservation in (response["Reservations"]):
        for instance in reservation["Instances"]:
            instancelist.append(instance["InstanceId"])

    return instancelist


def configure_environment(action, environment):
    ec2resourcers = connect_to_ec2_resources()
    if len(environment) == 0:
        print("No instances found in the environment, check your tags and filters")
        return 1
    if action == WAKEUP:
        ec2resourcers.instances.filter(InstanceIds=environment).start()

        # wait till all instances start
        for instance in environment:
            ec2resourcers.Instance(instance).wait_until_running()
    elif action == SHUTDOWN:
        ec2resourcers.instances.filter(InstanceIds=environment).stop()
        # wait till all instances stop
        for instance in environment:
            ec2resourcers.Instance(instance).wait_until_stopped()
    return 0


def validate_env_status(action, filters):
    new_environment = list_instances_by_tag_value(action, filters)
    if len(new_environment) > 0:
        print_and_exit("Failed to {} environment, "
                       "\nsee instances with unknown status in AWS console:\n {}".format(action, create_aws_url(
            new_environment)), 2)
    else:
        print("environment {} successfully".format(action))

    return new_environment


def get_region():
    my_session = boto3.session.Session()
    return my_session.region_name


def create_aws_url(bad_instances):
    # open browser with instances
    base_url = "https://console.aws.amazon.com/ec2/v2/home"
    region_prefix = "?region="
    region = get_region()
    instances_search = "#Instances:search="
    instances_ids = ""
    sort = ";sort=instanceState"
    for bad_instance in bad_instances:
        instances_ids += bad_instance + ","
    url = base_url + region_prefix + region + instances_search + instances_ids + sort
    return url


def main(*args):
    action, filters = parse_args()
    environment = list_instances_by_tag_value(action=action, filters=filters)
    if not len(environment) > 0:
        print_and_exit("No instances found in the environment", 2)
    print("running on AWS region: {}".format(get_region()))
    print("{} called for the following instances: {}".format(action, environment))

    configure_environment(action, environment)
    validate_env_status(action=action, filters=filters)











'''
parse arguments from Jenkins/CLI
action:  required action on the environment: WAKEUP/SHUTDOWN
tags (list): key value pair of AWS tag name and tag value in the format: -t tag1-val1 -t tag2=val2...
:return:
1. action
2. list of tags
'''

import argparse
from configAWSEnv.const import WAKEUP, SHUTDOWN
from configAWSEnv.prints import print_and_exit


def parse_args():
    # get args: action and AWS instances tag filters
    parser = argparse.ArgumentParser(description='get the required action and AWS instance tags')
    parser.add_argument(
        '--action', '-a', metavar='string', type=str,
        help='\'WAKEUP\'/\'SHUTDOWN\' action to run on the AWS environment', required=True)
    my_dict = {}

    class StoreDictKeyPair(argparse.Action):
        def __call__(self, parser, namespace, values, option_string=None):
            for kv in values.split(","):
                k, v = kv.split("=")
                my_dict[k] = v
            setattr(namespace, self.dest, my_dict)

    parser.add_argument("--tags", "-t", dest="my_dict", action=StoreDictKeyPair, metavar="KEY1=VAL1,KEY2=VAL2...")
    args = parser.parse_args()
    if not args.action in (WAKEUP, SHUTDOWN):
        print_and_exit("Invalid action passed, only {} and {} are allowed".format(WAKEUP, SHUTDOWN), 2)
    action = args.action
    filters = []
    info = {}
    # set the filters according to bot standards
    for tag in my_dict:
        info = {
            'Name': 'tag:{}'.format(tag),
            'Values': [my_dict.get(tag)]
        }
        filters.append(info)

    return action, filters


#!/usr/bin/env python

from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def pre_install_script(self):
        pass

    def post_install_script(self):
        pass

    def run(self):
        self.pre_install_script()

        _install.run(self)

        self.post_install_script()

if __name__ == '__main__':
    setup(
        name = 'configAWSEnv',
        version = '0.0.4',
        description = 'A utility to stop and start AWS environments',
        long_description = 'This utility stops and starts AWS environments (group of instances according to tags) from an external scheduler',
        author = 'Ittiel',
        author_email = 'ittiel@gmail.com',
        license = 'Apache License',
        url = 'https://github.com/taykey/schedule-aws-env',
        scripts = ['scripts/configEnv.py'],
        packages = ['configAWSEnv'],
        namespace_packages = [],
        py_modules = ['__init__'],
        classifiers = [
            'Programming Language :: Python',
            'Programming Language :: Python :: Implementation :: CPython',
            'Programming Language :: Python :: Implementation :: PyPy',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Development Status :: 4 - Beta',
            'Environment :: Console',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: Apache Software License',
            'Topic :: Software Development :: Quality Assurance',
            'Topic :: Software Development :: Testing'
        ],
        entry_points = {
            'console_scripts': ['configAWSEnv=configAWSEnv:config_ec2_env.main']
        },
        data_files = [],
        package_data = {
            'pybuilder': ['LICENSE']
        },
        install_requires = [
            'argparse==1.4.0',
            'boto3==1.4.7'
        ],
        dependency_links = [],
        zip_safe = True,
        cmdclass = {'install': install},
        keywords = '',
        python_requires = '',
        obsoletes = [],
    )
